@@ -38,6 +38,7 @@ universal_arglist = {
     "direction"     : {DEFAULT : ">", DESC : "The direction for applying the threshold, > or <"},
     "double_ended"  : {DEFAULT : -1,   DESC : "For handling double-ended fibre [-1=single-ended, 0=start-of-fibre half, 1=end-of-fibre half"},
     "edge_order"    : {DEFAULT :  1,   DESC : "Gradient is calculatd using N-th order accurate differences at the boundaries"},
+    "filename"      : {DEFAULT : NONE, DESC : "A filename, the complete path will be defined by directory_out and project configurations"},
     "command_list" :  {                DESC : "A list of sub-commands collected in a single macro"},
     "train" :         {                DESC : "A dictionary containing training parameters for keras, e.g. { 'epochs' : 150, 'batch_size' : 10 }"},
     "func" :          {                DESC : "Either rms_from_fft or te_from_fft"}, 
@@ -48,6 +49,7 @@ universal_arglist = {
     "prf" :           {DEFAULT : 10000, DESC : "The pulse repetition frequency in Hz (one over the time sample rate)"},
     "freq" :          {DEFAULT : 200.0, DESC : "A frequency in Hz or a wavenumber in 1/m"},
     "filename" :      {DEFAULT : NONE,  DESC : "A filename for read or write operations"},
+    "index" :         {DEFAULT : 0,     DESC : "The index of a row or column in sanmples"},
     "axis" :          {DEFAULT : -1,    DESC : "The axis to apply an operation to, typically in distpy axis=0 for depth and axis=1 for time"},
     "xsigma" :        {DEFAULT : 1.0,   DESC : "A standard deviation in the x-direction"},
     "tsigma" :        {DEFAULT : 1.0,   DESC : "A standard deviation in the time direction"},
@@ -70,7 +72,14 @@ universal_arglist = {
     "moment" :        {DEFAULT : 1,     DESC : "The order of central moment, see scipy.stats.moment"},
     "max_velocity":   {DEFAULT : 1600,  DESC : "The maximum phase velocity"},
     "min_velocity":   {DEFAULT : 1400,  DESC : "The minimum phase velocity"},
+    "velocity":       {DEFAULT : 1400,  DESC : "Phase velocity"},
+    "bandwidth":      {DEFAULT : 0.1,   DESC : "Width of the Gaussian ray in pixels"},
+    "shape":          {DEFAULT : [9,9], DESC : "Shape of the filter in pixels e.g. [9,9], both values must be even"},
+    "max_val":        {DEFAULT : 1.0,   DESC : "An upper bound"},
+    "min_val":        {DEFAULT : 0.0,   DESC : "A lower bound"},
     "smooth" :        {DEFAULT : 0, DESC : "The smoothing factor for the filter"},
+    "threshold" :     {DEFAULT : 0, DESC : "An upper or lower limit"},
+    "value" :         {DEFAULT : 0, DESC : "A single floating point number"},
     "bounds" :        {DEFAULT : ([-5,0],[0,400]), DESC : "The bounds on curve fitting, see scipy.optimize.curve_fit"},
     "initSlopes" :    {DEFAULT : [-0.5], DESC : "An initial slope estimate for the curve fitting, see scipy.optimize.curve_fit"},
     "xsample" :       {DEFAULT : 1,     DESC : "The level of downsampling in the x-directioin"},
@@ -270,17 +279,22 @@ class AngleCommand(BasicCommand):
 class RescaleCommand(BasicCommand):
     def __init__(self,command, jsonArgs):
         super().__init__(command, jsonArgs)
+        self._prevstack = jsonArgs.get('commands',[None])
 
     def docs(self):
         docs={}
-        docs['one_liner']="Rescale the data from 0 to 1"
+        docs['one_liner']="Rescale the data from 0 to 1. If an additional data set it suppied, the rescale uses that data for min() and max()"
         return docs
 
     def isGPU(self):
         return True
 
     def execute(self):
-        self._result = extra_numpy.agnostic_rescale(self._previous.result())
+        Y = None
+        if len(self._prevstack)>0:
+            if self._prevstack[0] is not None:
+                Y = self._prevstack[0].result()
+        self._result = extra_numpy.agnostic_rescale(self._previous.result(), external_scaling=Y)
 '''
  SoftThresholdCommand : applies a threshold greater or less
 '''
@@ -288,18 +302,63 @@ class SoftThresholdCommand(BasicCommand):
     def __init__(self,command, jsonArgs):
         super().__init__(command, jsonArgs)
         self._direction = jsonArgs.get('direction','>')
+        self._threshold = jsonArgs.get('threshold',0.0)
 
     def docs(self):
         docs={}
-        docs['one_liner']="Applies a soft threshold."
-        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['direction'] }
+        docs['one_liner']="Applies a soft threshold, clipping the values at the given threshold."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['direction','threshold'] }
         return docs
 
     def isGPU(self):
         return True
 
     def execute(self):
-        self._result = extra_numpy.soft_threshold(self._previous.result(),direction=self._direction)
+        self._result = extra_numpy.soft_threshold(self._previous.result(),self._threshold,direction=self._direction)
+
+'''
+ PeakBroadeningCommand : broadens any peaks by duplicating the maximum value
+'''
+class PeakBroadeningCommand(BasicCommand):
+    def __init__(self,command, jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._window_length = jsonArgs.get('window_length',50)
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="Broadening of the local maxima, by extending them in time."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['window_length'] }
+        return docs
+
+    def isGPU(self):
+        return True
+
+    def execute(self):
+        self._result = extra_numpy.broaden(self._previous.result(),window_length=self._window_length)
+
+
+'''
+ HardThresholdCommand : applies a threshold greater or less
+'''
+class HardThresholdCommand(BasicCommand):
+    def __init__(self,command, jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._direction = jsonArgs.get('direction','>')
+        self._threshold = jsonArgs.get('threshold',0.0)
+        self._value     = jsonArgs.get('value',0.0)
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="Applies a hard threshold, all values beyond the threshold are replaced with the supplied value."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['direction','threshold','value'] }
+        return docs
+
+    def isGPU(self):
+        return True
+
+    def execute(self):
+        self._result = extra_numpy.hard_threshold(self._previous.result(),self._threshold, self._value,direction=self._direction)
+
 
 
 '''
@@ -404,6 +463,8 @@ class KMeansCommand(BasicCommand):
 
     def execute(self):
         n_clusters = self._args.get('n_clusters',10)
+        if len(self._prevstack) < 1:
+            self._prevstack = [None]
         if self._prevstack[0] is not None:
             shape_out = self._previous.result().shape
             for a in range(len(self._prevstack)):
@@ -684,7 +745,7 @@ class GaussianCommand(BasicCommand):
         return docs
 
     def execute(self):
-        self._result = ndimage.gaussian_filter(self._previous.result(), [self._xdir, self._tdir], order=[self._xorder, self._torder],mode=elf._mode)
+        self._result = ndimage.gaussian_filter(self._previous.result(), [self._xdir, self._tdir], order=[self._xorder, self._torder],mode=self._mode)
 
 '''
  SobelCommand : Applies a Sobel edge detection filter using signal.ndarray.sobel()
@@ -721,7 +782,33 @@ class ConvolveCommand(BasicCommand):
         return docs
 
     def execute(self):
-        self._result = ndimage.convolve(self._previous.result(),self._coeffs, mode=self._mode)
+        self._result = extra_numpy.convolve2D(self._previous.result(),self._coeffs, mode=self._mode)
+
+'''
+ TXDipCommand : Gaussian-ray dip filter in the x-t domain
+'''
+class TXDipCommand(BasicCommand):
+    def __init__(self,command, jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._args=jsonArgs
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="time-space domain dip filter."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['velocity','bandwidth','shape'] }
+        docs['args']['velocity']['default'] = 100 
+        docs['args']['bandwidth']['default'] = 0.1 
+        docs['args']['shape']['default'] = [9,9] 
+        return docs
+
+    def execute(self):
+        dx = numpy.abs(self._args['xaxis'][1]-self._args['xaxis'][0])
+        dt = 1.0/self._args['prf']
+        speed = self._args.get('velocity',100)
+        bandwidth = self._args.get('bandwidth',0.1)
+        boxshape = self._args.get('shape',[9,9])
+        
+        self._result = extra_numpy.tx_dipfilter(self._previous.result(),dx,dt,speed,bandwidth=0.1,width=9,height=9)
 
 '''
  CorrelateCommand : Applies the supplied correlation filter
@@ -791,10 +878,11 @@ class RollCommand(BasicCommand):
         super().__init__(command, jsonArgs)
         self._axis  = jsonArgs.get('axis',1)
         self._shift = jsonArgs.get('window_length',1)
+        self._prevstack = jsonArgs.get('commands',[None])
 
     def docs(self):
         docs={}
-        docs['one_liner']="Rolls the data along the specified axis, using numpy.roll()."
+        docs['one_liner']="Rolls the data along the specified axis, using numpy.roll(). An array can be passed in through gather_uid"
         docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['axis','window_length'] }
         docs['args']['axis']['default']=1
         docs['args']['window_length']['default']=1
@@ -804,7 +892,20 @@ class RollCommand(BasicCommand):
         return True
 
     def execute(self):
-        self._result = extra_numpy.agnostic_roll(self._previous.result(), self._shift, axis=self._axis)
+        Y = None
+        if len(self._prevstack)>0:
+            if self._prevstack[0] is not None:
+                Y = self._prevstack[0].result()
+        if Y is not None:
+            self._result = numpy.zeros(((self._previous.result()).shape))
+            if self._axis==1:
+                for a in range((self._previous.result()).shape[0]):
+                    self._result[a,:]=extra_numpy.agnostic_roll((self._previous.result())[a,:], Y[a])
+            else:
+                for a in range((self._previous.result()).shape[1]):
+                    self._result[:,a]=extra_numpy.agnostic_roll((self._previous.result())[:,a], Y[a])
+        else:
+            self._result = extra_numpy.agnostic_roll(self._previous.result(), self._shift, axis=self._axis)
 
 
 
@@ -888,7 +989,10 @@ class ClipCommand(BasicCommand):
         self._tmin = jsonArgs.get('tmin',numpy.min(self._taxis))
         self._tmax = jsonArgs.get('tmax',numpy.max(self._taxis))
         self._double_ended = jsonArgs.get('double_ended',-1)
-        self._fname = str(int(self._tmin))
+        # for unit tests and documentation...
+        if self._tmin is None:
+            self._tmin=0
+        self._fname = str(numpy.int(self._tmin))
 
     def docs(self):
         docs={}
@@ -966,14 +1070,15 @@ class GradientCommand(BasicCommand):
     def __init__(self,command, jsonArgs):
         super().__init__(command, jsonArgs)
         self._xaxis = jsonArgs['xaxis']
-        self._taxis = jsonArgs['taxis']
+        self._taxis = jsonArgs.get('taxis',None)            
         self._edge_order = jsonArgs.get('edge_order',1)
-        self._axis = jsonArgs.get('axis',None)
+        self._axis = jsonArgs.get('axis',0)
 
     def docs(self):
         docs={}
         docs['one_liner']="Numercal gradient of the data via central differencing."
         docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['edge_order','axis'] }
+        docs['args']['axis']['default']=0
         return docs
 
     def isGPU(self):
@@ -1071,6 +1176,7 @@ class MultiplyCommand(BasicCommand):
         return True
 
     def execute(self):
+        
         if self._prevstack[0]!=None:
             self._result = self._previous.result()*((self._prevstack[0]).result())
 '''
@@ -1140,6 +1246,34 @@ class GatherCommand(BasicCommand):
                 datalist.append(dataset.result())
             self._result = extra_numpy.gather(self._previous.result(),datalist)
 
+'''
+ ExtractCommand : Extract a named column or row
+'''
+class ExtractCommand(BasicCommand):
+    def __init__(self,command,jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._prevstack = jsonArgs.get('commands',[None])
+        self._idx = jsonArgs.get('index',0)
+        self._axis = jsonArgs.get('axis',1)
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="Extracts a single row or column as a separate dataset"
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['index','axis'] }
+        docs['args']['axis']['default'] = 1
+        return docs
+
+    def isGPU(self):
+        return True
+
+    def execute(self):
+        if self._axis==0:
+            self._result = (self._previous.result())[self._idx,:]
+        else:
+            self._result = (self._previous.result())[:,self._idx]
+        self._result = numpy.reshape(self._result,(self._result.flatten().shape[0],1))
+
+
 '''
  StaLtaCommand : short-term average to long-term-average onset-picker transform
 '''
@@ -1189,6 +1323,9 @@ class WriteNPYCommand(BasicCommand):
         super().__init__(command, jsonArgs)
         self._dirname = jsonArgs.get('directory_out','NONE')
         self._fname = jsonArgs.get('date_dir','NONE')
+        self._xaxis = jsonArgs['xaxis']
+        self._taxis = jsonArgs.get('taxis',None)            
+
 
     def docs(self):
         docs={}
@@ -1199,11 +1336,43 @@ class WriteNPYCommand(BasicCommand):
     def postcond(self):
         return [directory_services.path_join(self._dirname,self._fname+'.npy')]
 
+    def isGPU(self):
+        return True
+
     def execute(self):
         super().execute()
         dirname = self._dirname
         if not dirname=='NONE':
-            io_helpers.numpy_out(dirname,self._fname,self._previous.result())
+            extra_numpy.agnostic_save(dirname,self._fname,self._previous.result())
+            extra_numpy.agnostic_save(dirname,'measured_depth',self._xaxis)
+            if self._taxis is not None:
+                extra_numpy.agnostic_save(dirname,'time',self._taxis)
+
+'''
+ WriteNPYCommand : writes out the current datablock to NPY format
+'''
+class ReadNPYCommand(BasicCommand):
+    def __init__(self, command, jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._dirname = jsonArgs.get('directory_out','NONE')
+        self._fname = jsonArgs.get('filename','NONE')
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="Load a npy format file."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['directory_out','filename'] }
+        return docs
+
+    def isGPU(self):
+        return True
+
+    def execute(self):
+        super().execute()
+        dirname = self._dirname
+        if not dirname=='NONE':
+            # we can use the previous result to tell us whether we are on GPU or CPU
+            self.result = extra_numpy.agnostic_load(dirname,self._fname,self._previous.result())
+
 
 '''
  WriteWITSMLCommand : writes the results to the WITSML/FBE format.
@@ -1429,6 +1598,26 @@ class VelocityMaskCommand(BasicCommand):
         smoothval = self._args.get('smooth',0)
         self._result = extra_numpy.vel_mask(velmap,minv,maxv,smoothval)
 
+'''
+ BoundedSelectCommand : return a mask within a pair of bounds on the data
+'''
+class BoundedSelectCommand(BasicCommand):
+    def __init__(self, command, jsonArgs):
+        super().__init__(command, jsonArgs)
+        self._args = jsonArgs
+
+    def docs(self):
+        docs={}
+        docs['one_liner']="Return a masks within a pair of bounds on the data (e.g. selecting a cluster from k-means results)."
+        docs['args'] = { a_key: universal_arglist[a_key] for a_key in ['min_val','max_val'] }
+        return docs
+
+    def execute(self):
+        velmap = self._previous.result()
+        minv = self._args.get('min_val',0.0)
+        maxv = self._args.get('max_val', 1.0)
+        self._result = extra_numpy.bounded_select(velmap,minv,maxv)
+
 '''
  MultipleCalcsCommand : allows multiple calculations to be made using the reduced_mem form.  
 '''
@@ -1483,16 +1672,21 @@ def KnownCommands(knownList):
     knownList['analytic_signal']= AnalyticSignalCommand
     knownList['argmax']         = ArgmaxCommand
     knownList['approx_vlf']     = ApproxVlfCommand
+    knownList['bounded_select'] = BoundedSelectCommand
+    knownList['broaden']        = PeakBroadeningCommand
     knownList['butter']         = ButterCommand
     knownList['clip']           = ClipCommand
     knownList['conj']           = ConjCommand
     knownList['convolve']       = ConvolveCommand
     knownList['correlate']      = CorrelateCommand
     knownList['count_peaks']    = CountPeaksCommand
+    knownList['data_load']      = ReadNPYCommand
     knownList['destripe']       = DestripeCommand
     knownList['diff']           = DiffCommand
     knownList['downsample']     = DownsampleCommand
+    knownList['dip_filter']     = TXDipCommand
     knownList['down_wave']      = DownCommand
+    knownList['extract']        = ExtractCommand
     knownList['fft']            = FFTCommand
     knownList['from_gpu']       = FromGPUCommand
     knownList['gather']         = GatherCommand
@@ -1500,6 +1694,7 @@ def KnownCommands(knownList):
     knownList['geometric_mean'] = GeometricMeanCommand
     knownList['gradient']       = GradientCommand
     knownList['harmonic_mean']  = HarmonicMeanCommand
+    knownList['hard_threshold'] = HardThresholdCommand
     knownList['ifft']           = IFFTCommand
     knownList['keras']          = KerasCommand
     knownList['kmeans']         = KMeansCommand
@@ -1533,7 +1728,7 @@ def KnownCommands(knownList):
     return knownList    
 
 '''
- tesxt_for_users : A header and introduction for autogenerated documentation.
+ text_for_users : A header and introduction for autogenerated documentation.
 '''
 def text_for_users():
     text_for_users = {}
