@@ -1,6 +1,7 @@
 from __future__ import division, absolute_import, print_function
 
-__all__ = ['parallel_convert_las2fbe',
+__all__ = ['parallel_ingest',
+           'parallel_convert_las2fbe',
            'parallel_ingest_csv',
            'parallel_ingest_h5',
            'parallel_ingest_sgy',
